/*
 * @Author       : stark1898y 1658608470@qq.com
 * @Date         : 2024-12-15 16:13:37
 * @LastEditors  : stark1898y 1658608470@qq.com
 * @LastEditTime : 2025-05-23 18:50:01
 * @FilePath     : \IoT_SCV_CH584M\bsp\src\bsp_iwdg.c
 * @Description  :
 *
 * Copyright (c) 2024 by yzy, All Rights Reserved.
 */
// https://www.cnblogs.com/debugdabiaoge/p/17580033.html
// https://www.cnblogs.com/risc5-ble/p/17853714.html
#include "bsp_iwdg.h"
#include "bsp_uart.h"
#include "log.h"

#undef LOG_ENABLE
#define LOG_ENABLE 1

#define LOG_LVL    LOG_DEBUG

#undef LOG_TAG
#define LOG_TAG "idwg"

uint16_t _timeout_ms, _feed_ms;

tmosTaskID iwdg_task_id = INVALID_TASK_ID;

uint16_t IWDG_ProcessEvent(uint8_t task_id, uint16_t events)
{
    if (events & IWIG_FEED_EVENT)
    {
        FEED_IWDG();
        // logDebug("IWDG feed");
        // 获取tmos系统堆的剩余内存大小
        // uint8_t* a = tmos_msg_allocate(1);
        logDebug("tmos_memory_getlen = %d" , tmos_memory_getlen());
        tmos_start_task(task_id, IWIG_FEED_EVENT, MS1_TO_SYSTEM_TIME(_feed_ms));
        return (events ^ IWIG_FEED_EVENT);
    }
    return 0;
}

// 32k rc
// div 0:4  | 1:8   | 2:16  | 3:32  |
//     4:64 | 5:128 | 6:256 | 7:512(32K分频512，时钟62.5Hz) |
// reload max 4095

// 独立看门狗（IWDG）由专用的内部低速时钟（LSI）驱动，能够在低功耗模式下正常工作。
// 根据寄存器描述，IWDG最大时间可以是(0xFFF/(32K/512))=65.52s
void IWDG_Init(uint16_t timeout_ms, uint16_t feed_ms)
{
    uint16_t div = 512;
    // float tick_ms = 1000 / (32000 / 512);
    uint8_t tick_ms = 16;

    uint16_t reload = (timeout_ms / tick_ms) + 1;

    // 解除IWDG保护
    R32_IWDG_KR  = 0x5555;

    R32_IWDG_CFG |= (7 << 12); // 32K分频512，时钟62.5Hz
    R32_IWDG_CFG &= 0xFFFFF000;
    // R32_IWDG_CFG |= 0x7D * 3; // 看门狗时间约6 s（0x10/(32K/512))
    R32_IWDG_CFG |= reload;

    // 开启IWDG保护
    R32_IWDG_KR  = 0xCCCC;
    // uint32_t d = R32_IWDG_CFG;
    // logHexDumpAll(d, 4);

    // TODO:这里一算就出问题
    float a = (((float)reload * 1000) / 62.5);
    _timeout_ms = (uint16_t)(a);
    _feed_ms = feed_ms;

    logDebug("timeout_ms: %d, feed_ms: %d", _timeout_ms, _feed_ms);

    // sys_safe_access_enable();
    // R8_SAFE_LRST_CTRL |= RB_IWDG_RST_EN;
    // sys_safe_access_disable();

    iwdg_task_id = TMOS_ProcessEventRegister(IWDG_ProcessEvent);
    if (iwdg_task_id == INVALID_TASK_ID)
    {
        logError("IWDG task register failed");
        return;
    }
    // tmos_set_event(iwdg_task_id, IWIG_FEED_EVENT);
    tmos_start_task(iwdg_task_id, IWIG_FEED_EVENT, MS1_TO_SYSTEM_TIME(_feed_ms));
    logInfo("iwdg_task_id = %d", iwdg_task_id);
}

#define  RB_RLR             0x0FFF                    // RW, watch-dog counter reload (write protect)
#define  RB_PR              0x7000                    // PR, prescale (write protect)
#define  RB_PVU             0x8000                    // RO, register update flag (write protect)
#define  RB_COUNT           0xFF0000                  // RO, watch-dog down counter
#define  RB_STOP_EN         0x20000000                // RW, watch-dog stop enable (write protect)
#define  RB_WR_PROTECT      0x40000000                // RO, write protect
#define  RB_IWDG_EN         0x80000000                // RO, watch-dog enable

// 打印复位状态寄存器，显示复位原因
// 最近一次复位状态：
// 000：软件复位 SR（RB_WDOG_RST_EN=0时软件复位可产生此状态，否则可复位但不产生此状态）；
// 001：上电复位 RPOR；
// 010：看门狗超时复位 WTR；
// 011：外部手动复位 MR；
// 101：从下电模式唤醒时的复位 GRWSM；
// 100/110/111：唤醒复位 LRW，且此前的上一次复位分别是 SR/WTR/MR。
void ShowRestartReason(void)
{
    uint8_t reason = R8_RESET_STATUS&0x07;
    logDebug("R8_RESET_STATUS = %02x", reason);
    switch (reason)
    {
    case RST_FLAG_SW:
        logDebug("RST_FLAG_SW");
        break;
    case RST_FLAG_RPOR:
        logDebug("RST_FLAG_RPOR");
        break;
    case RST_FLAG_WTR:
        logDebug("RST_FLAG_WTR");
        break;
    case RST_FLAG_MR:
        logDebug("RST_FLAG_MR");
        break;

    case RST_FLAG_GPWSM:
        logDebug("RST_FLAG_GPWSM");
        break;

    case 0x04:
        logDebug("LRW, last SR");
        break;
    case 0x06:
        logDebug("LRW, last WTR");
        break;
    case 0x07:
        logDebug("LRW, last MR");

    default:
        break;
    }
}
